package com.greenorange.promotion.model.dto.courseOrder;

import com.greenorange.promotion.annotation.EnumValue;
import com.greenorange.promotion.model.enums.OrderStatusEnum;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Min;
import lombok.Data;
import java.math.BigDecimal;

import java.io.Serial;
import java.io.Serializable;
import com.greenorange.promotion.common.PageRequest;
import lombok.EqualsAndHashCode;

/**
 * 课程订单查询请求体，继承自分页请求 PageRequest
 */
@Data
@Schema(description = "课程订单查询请求体", requiredProperties = {"current", "pageSize"})
public class CourseOrderQueryRequest extends PageRequest implements Serializable {

    /**
     * 订单号
     */
    @Schema(description = "订单号", example = "202506241339232334d234234243")
    private String orderNumber;


    /**
     * 订单状态[交易关闭,交易成功,待支付,已退款]
     */
    @EnumValue(enumClass = OrderStatusEnum.class)
    @Schema(description = "订单状态[交易关闭,交易成功,待支付,已退款]", example = "交易成功")
    private String orderStatus;


    @Serial
    private static final long serialVersionUID = 1L;
}

