package com.greenorange.promotion.model.dto.courseOrder;

import com.greenorange.promotion.annotation.EnumValue;
import com.greenorange.promotion.model.enums.OrderStatusEnum;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Min;
import lombok.Data;
import java.math.BigDecimal;

import java.io.Serial;
import java.io.Serializable;

/**
 * 课程订单更新请求体
 */
@Data
@Schema(description = "课程订单更新请求体", requiredProperties = {
                "id",
                "orderStatus"
})
public class CourseOrderUpdateRequest implements Serializable {

    /**
     * 订单ID
     */
    @Min(value = 1L, message = "订单ID ID不能小于1")
    @Schema(description = "订单ID", example = "1")
    private Long id;


    /**
     * 订单状态[交易关闭,交易成功,待支付,已退款]
     */
    @NotBlank(message = "订单状态不能为空")
    @EnumValue(enumClass = OrderStatusEnum.class)
    @Schema(description = "订单状态[交易关闭,交易成功,待支付,已退款]", example = "交易成功")
    private String orderStatus;


    @Serial
    private static final long serialVersionUID = 1L;
}