package com.greenorange.promotion.model.dto.promoCodeApply;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.Min;
import lombok.Data;
import java.math.BigDecimal;

import java.io.Serial;
import java.io.Serializable;
import com.greenorange.promotion.common.PageRequest;

/**
 * 推广码申请记录查询请求体，继承自分页请求 PageRequest
 */
@Data
@Schema(description = "推广码申请记录查询请求体", requiredProperties = {"current", "pageSize"})
public class PromoCodeApplyQueryRequest extends PageRequest implements Serializable {

    /**
     * 推广码申请ID
     */
    @Min(value = 1L, message = "推广码申请ID ID不能小于1")
    @Schema(description = "推广码申请ID", example = "1")
    private Long id;

    /**
     * 业务员姓名
     */
    @Schema(description = "业务员姓名", example = "chenxinzhi")
    private String salespersonName;

    /**
     * 业务员手机号
     */
    @Schema(description = "业务员手机号", example = "15888610253")
    private String salespersonPhone;

    /**
     * 绑定的推广码信息key
     */
    @Schema(description = "绑定的推广码信息key", example = "ykxy227")
    private String promoCodeInfoKey;

    /**
     * 绑定的推广码链接
     */
    @Schema(description = "绑定的推广码链接", example = "mp://y04CZgGkCBmIaZd")
    private String promoCodeLink;

    /**
     * 推广码图片
     */
    @Schema(description = "推广码图片", example = "http://xxx.png")
    private String promoCodeImage;

    /**
     * 绑定的项目名称
     */
    @Schema(description = "绑定的项目名称", example = "美团省钱包")
    private String projectName;

    /**
     * 绑定的项目图片URL
     */
    @Schema(description = "绑定的项目图片URL", example = "3E8U2AM8")
    private String projectImage;

    /**
     * 用户ID
     */
    @Min(value = 1L, message = "用户ID ID不能小于1")
    @Schema(description = "用户ID", example = "1")
    private Long userId;


    @Serial
    private static final long serialVersionUID = 1L;
}

