package com.greenorange.promotion.model.dto.withdrawalApply;

import com.greenorange.promotion.annotation.EnumValue;
import com.greenorange.promotion.model.enums.WithdrawStatusEnum;
import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.Min;
import lombok.Data;
import java.math.BigDecimal;

import java.io.Serial;
import java.io.Serializable;

/**
 * 提现申请记录更新请求体
 */
@Data
@Schema(description = "提现申请记录更新请求体", requiredProperties = {
                "id",
                "withdrawalStatus"
})
public class WithdrawalApplyUpdateRequest implements Serializable {

    /**
     * 提现申请ID
     */
    @Min(value = 1L, message = "提现申请ID ID不能小于1")
    @Schema(description = "提现申请ID", example = "1")
    private Long id;

    /**
     * 提现状态[提现中(processing)|提现成功(success)|提现失败(failed)]
     */
    @EnumValue(enumClass = WithdrawStatusEnum.class)
    @Schema(description = "提现状态[提现中(processing)|提现成功(success)|提现失败(failed)]", example = "processing")
    private String withdrawalStatus;


    @Serial
    private static final long serialVersionUID = 1L;
}